package jaxt.examples.position;

import static junit.framework.Assert.*;

/**
 * Axioms for the Position class. These are the specific requirements for the
 * class, i.e., those not already covered by Object and Comparable, its two
 * supertypes.
 * 
 * @author Magne Haveraaen, 2007
 * 
 */

abstract public class PositionAxioms extends Position implements
		jaxt.framework.Axioms {

	/**
	 * Project specific requirement: equals is a congruence.
	 */
	public static void congruencePlus(Position p1, Position p2, Position q1,
			Position q2) {
		if (p1 == p2 || p1 == q2)
			return; // same object
		if (p1.equals(p2) && q1.equals(q2)) {
			p1.plus(q1); // destructive update
			p2.plus(q2); // destructive update
			assertEquals(p1, p2);
		}
	}

	/**
	 * Plus is a commutative monoid, hence associative.
	 */
	public static void plusAssociative(Position p, Position q, Position r) {
		if (p == q)
			return;
		Position pc = new Position(p.getX(), p.getY());
		// pc = (p + q) + r
		pc.plus(q);
		pc.plus(r);
		// p = p + (q + r)
		q.plus(r); // destructive update
		p.plus(q); // destructive update
		assertEquals(pc, p);
	}

	/**
	 * Plus is a commutative monoid, hence commutative.
	 */
	public static void plusCommutative(Position p, Position q) {
		Position pc = new Position(p.getX(), p.getY());
		pc.plus(q);
		q.plus(p); // destructive update
		assertEquals(pc, q);
	}

	/**
	 * Plus is a commutative monoid, hence has a neutral element.
	 */
	public static void plusNeutral(Position p) {
		Position q = new Position(0, 0);
		q.plus(p);
		assertEquals(p, q);
	}

	/**
	 * Since there is no constructor Position() we are forced to declare a
	 * constructore based on an existing constructor of the superclass. We
	 * declare this mock constructor private.
	 */
	private PositionAxioms() {
		super(0, 0);
	}
}
