package jaxt.java.lang;

import static junit.framework.Assert.assertEquals;

/**
 * Project specific recommendations as axioms for the java interface
 * java.lang.Comparable related to <a
 * url="http://java.sun.com/j2se/1.5.0/docs/api/">Java 5.0 API</a>
 * 
 * @see java.lang.Comparable
 * @see java.lang.Object
 * @author Magne Haveraaen & Karl Trygve Kalleberg, 2007
 * 
 */

abstract public class ComparableAxiomsProject implements jaxt.framework.Axioms,
		Comparable {

	/**
	 * Sees the equals method as a congruence relation (smallest equivalence
	 * relation that yields full abstraction). This should generate congruence
	 * axioms for every declared interface and class.
	 * <p>
	 * This axiom is less restrictive than the strong recommendation that the
	 * natural ordering is consistent with equals.
	 */
	public static void congruenceCompareTo(Comparable a, Comparable b) {
		try {
			if (a.equals(b))
				assertEquals(a.compareTo(b), 0);
		} catch (RuntimeException e) {
			// some run-time exception occurred
		}
	}

}